package dev.pfaff.glass;

import java.util.function.Function;

/**
 * Runtime configuration utilities.
 */
public final class Config {
	/**
	 * Parses the value of the environment variable {@code name} as a boolean where {@code false} is parsed from
	 * {@code 0} and {@code true} is parsed from {@code 1}.
	 *
	 * @param name         the name of the environment variable.
	 * @param defaultValue the value that should be returned if the environment variable is unset.
	 * @return {@code true} if the variable is set to {@code 1}, {@code false} if the variable is set to {@code 0}, or
	 * {@code defaultValue} otherwise.
	 */
	public static boolean getEnvFlag(String name, boolean defaultValue) {
		return switch (System.getenv(name)) {
			case "1" -> true;
			case "0" -> false;
			case null -> defaultValue;
			default -> {
				throw new IllegalArgumentException(STR."""
				Environment variable \{name} must be set to '0', or '1'. \
				If unset, it will default to '\{defaultValue ? "1" : "0"}'.""");
			}
		};
	}

	public static String getEnvString(String name, String defaultValue) {
		var s = System.getenv(name);
		if (s == null) return defaultValue;
		return s;
	}

	public static int getEnvInt(String name, int defaultValue) {
		var s = System.getenv(name);
		if (s == null) return defaultValue;
		try {
			return Integer.parseInt(s);
		} catch (NumberFormatException e) {
			throw new IllegalArgumentException(STR."""
			Environment variable \{name} must be set to a valid 32-bit integer. \
			If unset, it will default to \{defaultValue}.""");
		}
	}

	public static long getEnvLong(String name, long defaultValue) {
		var s = System.getenv(name);
		if (s == null) return defaultValue;
		try {
			return Long.parseLong(s);
		} catch (NumberFormatException e) {
			throw new IllegalArgumentException(STR."""
			Environment variable \{name} must be set to a valid 64-bit integer. \
			If unset, it will default to \{defaultValue}.""");
		}
	}

	public static <T> T getEnvT(String name, T defaultValue, Function<String, T> parser) {
		var s = System.getenv(name);
		if (s == null) return defaultValue;
		return parser.apply(s);
	}
}
